package mvd.jester.simulator.model;

import org.jgrapht.experimental.dag.DirectedAcyclicGraph;
import org.jgrapht.graph.DefaultEdge;
import mvd.jester.model.DagTask;
import mvd.jester.model.Subtask;

public class JobContext {

    private final DagTask task;
    private final long releaseTime;
    private final long deadline;
    private final Subtask source;
    // private long activeThreads; // TODO: Use threads

    public JobContext(DagTask task, long releaseTime) {
        this.task = task;
        this.releaseTime = releaseTime;
        this.deadline = releaseTime + task.getDeadline();
        this.source = findSource(task);
        // this.activeThreads = task.getNumberOfThreads();
    }

    private Subtask findSource(DagTask task) {
        DirectedAcyclicGraph<Subtask, DefaultEdge> jobDag = task.getJobDag();
        for (Subtask subtask : jobDag) {
            if (jobDag.inDegreeOf(subtask) == 0) {
                return subtask;
            }
        }
        throw new RuntimeException("No source found in DAG. How can that be!?");
    }

    /**
     * @return the task
     */
    public DagTask getTask() {
        return task;
    }

    /**
     * @return the source
     */
    public Subtask getSource() {
        return source;
    }

    /**
     * @return the deadline
     */
    public long getDeadline() {
        return deadline;
    }

    /**
     * @return the releaseTime
     */
    public long getReleaseTime() {
        return releaseTime;
    }

    public boolean checkRemainingThreads() {
        return false;
    }

}
